<?php

namespace FiloBlu\Refilo\Model\Adapter;

use FiloBlu\Refilo\Helper\Data as HelperData;
use Elasticsearch\ClientBuilder;

class Elastic extends AbstractAdapter
{
    /**
     * @var HelperData
     */
    private $helperData;

    /**
     * Mongo constructor.
     * @param HelperData $helperData
     */
    public function __construct(
        HelperData $helperData
    ) {
        $this->helperData = $helperData;
        $this->connect();
    }

    public function connect()
    {
        $hosts = [
            $this->helperData->getConfigValue("elastic/general/elastic_host"),         // IP + Port
        ];

        $this->_connection = ClientBuilder::create()           // Instantiate a new ClientBuilder
        ->setRetries(2)
        ->setHosts($hosts)      // Set the hosts
        ->build();
    }

    /**
     * @param $key
     * @param $objects
     * @param $collection
     * @param bool $insertMode
     * @param $mongoDatabase
     * @return mixed|void
     */
    public function update($key, $objects, $collection, $insertMode, $mongoDatabase)
    {
        $i = 1;

        foreach ($objects as $object) {

            $params['body'][] = [
                'index' => [
                    '_index' => $collection,
                    '_id' => $object->{$key},
                    '_type' => "_doc",
                ]
            ];
            unset ($object->{$key});
            $params['body'][] = (array) $object;

            if ($i % 1000 == 0) {
                $responses = $this->_connection->bulk($params);

                // erase the old bulk request
                $params = ['body' => []];

                // unset the bulk response when you are done to save memory
                unset($responses);
            }
            $i++;

        }

        try {

            if (!empty($params['body'])) {
                $responses = $this->_connection->bulk($params);
                if (php_sapi_name() === 'cli') {
                    print_r($responses);
                }


            }



        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function dropCollection($collection, $mongoDatabase) {
        return true;
    }

    /**
     * @return mixed
     * @throws Exception
     */
    public function ensureIndex($store,$indexes_to_exclude)
    {

        $indexes = [];

        foreach ($indexes_to_exclude as $idx) {

            $indexes[$idx]["enabled"] = false;

        }

        $mapping = [
            "enabled" => true,
            '_source' => [
                'enabled' => true,
            ],
            'properties' => $indexes
    ];

        $params = [
            'index' => $store,
            'body' => [
                'settings' => [
                    'number_of_shards' => 1,
                    'number_of_replicas' => 1,
                    "index.mapping.total_fields.limit" => 1000
                ],
                'mappings' => ['_doc'=>$mapping]
            ]
        ];


        try {
            $this->_connection->indices()->create($params);
        } catch (\Exception $e) {

            $this->_connection->indices()->putMapping(['index' => $store, 'type' => '_doc', 'body' => $mapping]);
        }


    }
}
